@@ -8,6 +8,8 @@ import sys
 from os import path, chdir, getcwd
 import json
 import numpy as np
+
+from psmpa.default import default_psmpa1_ref_dir
 from psmpa.util import (system_call_check, make_output_dir, read_fasta,
                            read_phylip, write_fasta, write_phylip,
                            read_stockholm)
@@ -30,7 +32,7 @@ def place_seqs_pipeline(study_fasta,
     # Identify reference files to use. Note that model file will be different
     # depending on which placement pipeline is indicated.
     project_dir = path.dirname(path.abspath(__file__))
-    ref_dir = path.join(project_dir, "default_files", "bacteria", "pro_ref")
+    ref_dir = default_psmpa1_ref_dir
     ref_msa, tree, hmm, model = identify_ref_files(ref_dir, placement_tool)
 
     # Run hmmalign to align study sequences with reference MSA.
@@ -451,3 +453,4 @@ def run_sepp(tree: str, ref_msa_fastafile: str, study_msa_fastafile: str,
     system_call_check(sepp_command, print_command=print_cmds,
                       print_stdout=print_cmds, print_stderr=print_cmds)
 
+
