@@ -1,19 +1,13 @@
 #!/usr/bin/env python
 
-# __copyright__ = "Copyright 2018-2021, The PICRUSt Project"
-# __license__ = "GPL"
-# __version__ = "2.4.1"
-
-import imp
-
 
+from Bio import SeqIO
 import os
-from os import makedirs, chmod
+from os import makedirs
 from os.path import abspath, dirname, isdir, join, exists, splitext
 from collections import defaultdict
 from subprocess import call
 import argparse
-import stat
 import shutil as _shutil
 import weakref as _weakref
 import warnings as _warnings
@@ -89,7 +83,6 @@ def write_fasta(seq, outfile):
 
 
 def read_phylip(filename, check_input=True):
-
     '''Reads in Phylip formatted multiple-sequence alignment and stores in
     dictionary with each key as a sequence name and each value as the sequence.
     If check_input=True then will check whether the number and length of each
@@ -147,7 +140,6 @@ def write_phylip(seq, outfile):
 
         # Write header if this is first line.
         if seq_count == 0:
-
             seq_length = len(seq[s])
 
             out_phylip.write(str(len(seq)) + " " + str(seq_length) + "\n")
@@ -155,7 +147,6 @@ def write_phylip(seq, outfile):
             seq_count += 1
 
         if seq_length != len(seq[s]):
-
             # Throw error if sequence is unexpected length.
             raise SystemExit("Expected sequence " + s + " to be length " +
                              str(seq_length) + ", but was " +
@@ -239,7 +230,7 @@ def system_call_check(cmd, print_command=False, print_stdout=False,
         stderr_file = join(temp_dir, "stderr.txt")
 
         with open(stdout_file, "wb") as stdout_fh, \
-             open(stderr_file, "wb") as stderr_fh:
+                open(stderr_file, "wb") as stderr_fh:
 
             return_value = call(cmd, stdout=stdout_fh,
                                 stderr=stderr_fh)
@@ -273,7 +264,7 @@ def system_call_check(cmd, print_command=False, print_stdout=False,
         if print_stderr:
             print(stderr_log, file=sys.stderr)
 
-    return(return_value)
+    return (return_value)
 
 
 def make_output_dir(dirpath, strict=False):
@@ -296,8 +287,8 @@ def make_output_dir(dirpath, strict=False):
     try:
         makedirs(dirpath)
     except IOError as e:
-        err_str = "Could not create directory '%s'. Are permissions set " +\
-                  "correctly? Got error: '%s'" %e
+        err_str = "Could not create directory '%s'. Are permissions set " + \
+                  "correctly? Got error: '%s'" % e
         raise IOError(err_str)
 
     return dirpath
@@ -320,8 +311,8 @@ def generate_temp_filename(temp_dir=None, prefix="", suffix=""):
     if not temp_dir:
         temp_dir = tempfile._get_default_tempdir()
 
-    return(join(temp_dir, prefix +
-                next(tempfile._get_candidate_names()) + suffix))
+    return (join(temp_dir, prefix +
+                 next(tempfile._get_candidate_names()) + suffix))
 
 
 def read_seqabun(infile):
@@ -334,7 +325,7 @@ def read_seqabun(infile):
     if in_ext == ".biom":
         input_seqabun = biom.load_table(infile).to_dataframe(dense=True)
         input_seqabun.index.astype('str', copy=False)
-        return(input_seqabun)
+        return (input_seqabun)
 
     # Next check if input file is a mothur shared file or not by read in first
     # row only.
@@ -359,13 +350,13 @@ def read_seqabun(infile):
         input_seqabun.index.name = None
         input_seqabun = input_seqabun.transpose()
         input_seqabun.index.astype('str', copy=False)
-        return(input_seqabun)
+        return (input_seqabun)
     else:
         first_col = str(pd.read_csv(infile, sep="\t", nrows=0).columns[0])
         input_seqabun = pd.read_csv(filepath_or_buffer=infile, sep="\t",
                                     dtype={first_col: str}, low_memory=False)
         input_seqabun.set_index(first_col, drop=True, inplace=True)
-        return(input_seqabun)
+        return (input_seqabun)
 
 
 def three_df_index_overlap_sort(df1, df2, df3):
@@ -387,7 +378,7 @@ def three_df_index_overlap_sort(df1, df2, df3):
     df2 = df2.reindex(index=label_overlap)
     df3 = df3.reindex(index=label_overlap)
 
-    return(df1, df2, df3)
+    return (df1, df2, df3)
 
 
 def check_files_exist(filepaths):
@@ -428,7 +419,7 @@ def add_descrip_col(inputfile, mapfile, in_df=False):
     else:
         function_tab = pd.read_csv(inputfile, sep="\t", low_memory=False,
                                    dtype={'function': str, 'sequence': str})
-    
+
     map_tab = pd.read_csv(mapfile, sep="\t", index_col=0, header=None,
                           names=["function", "description"],
                           low_memory=False, dtype=object)
@@ -446,7 +437,7 @@ def add_descrip_col(inputfile, mapfile, in_df=False):
     function_tab.insert(1, "description", list(map_tab["description"]))
 
     # Add description column to function table and return.
-    return(function_tab)
+    return (function_tab)
 
 
 def convert_humann2_to_picrust2(infiles, outfile, stratified):
@@ -495,7 +486,7 @@ def convert_humann2_to_picrust2(infiles, outfile, stratified):
 
     # Output table with current index if table isn't stratified.
     if not stratified:
-        humann2_combined.to_csv(path_or_buf=outfile,  sep="\t",
+        humann2_combined.to_csv(path_or_buf=outfile, sep="\t",
                                 index_label=first_col)
 
     # Otherwise if the table is stratified remove all rows that are in
@@ -505,7 +496,8 @@ def convert_humann2_to_picrust2(infiles, outfile, stratified):
 
         original_col = list(humann2_combined.columns)
 
-        split_df = pd.DataFrame.from_records(list(humann2_combined.index.str.split('\\|', 1)), columns=[first_col, 'sequence'])
+        split_df = pd.DataFrame.from_records(list(humann2_combined.index.str.split('\\|', 1)),
+                                             columns=[first_col, 'sequence'])
 
         humann2_combined.reset_index(inplace=True)
 
@@ -513,9 +505,9 @@ def convert_humann2_to_picrust2(infiles, outfile, stratified):
 
         # Reorder columns.
         humann2_combined = humann2_combined.loc[:, [first_col, 'sequence'] +
-                                                original_col]
+                                                   original_col]
 
-        humann2_combined.to_csv(path_or_buf=outfile,  sep="\t", index=False)
+        humann2_combined.to_csv(path_or_buf=outfile, sep="\t", index=False)
 
 
 def convert_picrust2_to_humann2(infiles, outfolder, stratified):
@@ -543,7 +535,6 @@ def convert_picrust2_to_humann2(infiles, outfolder, stratified):
     else:
 
         if len(infiles) != 2:
-
             sys.exit('Stopping - expected two input files (stratified and '
                      'unstratified PICRUSt2 tables) when converting to '
                      'HUMAnN2 stratified format')
@@ -567,7 +558,6 @@ def convert_picrust2_to_humann2(infiles, outfolder, stratified):
             sys.exit('Stopping - exactly one stratified table should have been '
                      'input - found ' + str(strat_table_count) + '.')
 
-
         # Check that the first column labels are the same across both tables.
         if in_tab1.columns[0] != in_tab2.columns[0]:
             sys.exit('Stopping - label of first column does not match between '
@@ -582,8 +572,9 @@ def convert_picrust2_to_humann2(infiles, outfolder, stratified):
         def set_picrust2_tab_index(in_table):
 
             if 'sequence' in in_table.columns:
-                in_table.index = in_table.loc[:, in_table.columns[0]].str.cat(others=in_table.loc[:, in_table.columns[1]],
-                                                                              sep='|')
+                in_table.index = in_table.loc[:, in_table.columns[0]].str.cat(
+                    others=in_table.loc[:, in_table.columns[1]],
+                    sep='|')
                 in_table.drop(in_table.columns[0:2], axis=1, inplace=True)
             else:
                 in_table.index = in_table[in_table.columns[0]]
@@ -591,7 +582,7 @@ def convert_picrust2_to_humann2(infiles, outfolder, stratified):
 
             in_table.index.name = "combined"
 
-            return(in_table)
+            return (in_table)
 
         in_tab1 = set_picrust2_tab_index(in_tab1)
         in_tab2 = set_picrust2_tab_index(in_tab2)
@@ -607,7 +598,7 @@ def convert_picrust2_to_humann2(infiles, outfolder, stratified):
     else:
         sys.exit('Error: first column of input PICRUSt2 tables is \"' +
                  index_name + '\". One of \"pathway\", \"gene\" or '
-                 '\"function\" were expected.')
+                              '\"function\" were expected.')
 
     # Remove description column if it is present.
     if 'description' in in_tab.columns:
@@ -622,12 +613,12 @@ def convert_picrust2_to_humann2(infiles, outfolder, stratified):
     # Write each sample to a different file in the output folder.
     for sample_id in in_tab.columns:
         outfile = join(outfolder, sample_id + "_humann2-format.tsv")
-        
+
         # Subset to this sample only and remove all rows that are 0.
         tab_subset = in_tab[[sample_id]]
         tab_subset = tab_subset.loc[~(tab_subset == 0).all(axis=1)]
 
-        tab_subset.to_csv(path_or_buf=outfile,  sep="\t",
+        tab_subset.to_csv(path_or_buf=outfile, sep="\t",
                           index_label=first_col)
 
 
@@ -640,7 +631,7 @@ def convert_picrust2_to_humann2_merged(infiles, outfile):
 
     # List for keeping track name of first column in each file.
     infile_index_names = []
-        
+
     for infile in infiles:
 
         in_table = pd.read_csv(infile, sep="\t", low_memory=False,
@@ -692,7 +683,7 @@ def convert_picrust2_to_humann2_merged(infiles, outfile):
                  '\"# Gene Family\" was expected.')
 
     # Write out.
-    new_tab.to_csv(path_or_buf=outfile,  sep="\t", index_label=first_col)
+    new_tab.to_csv(path_or_buf=outfile, sep="\t", index_label=first_col)
 
 
 def contrib_to_legacy(infiles, outfile, use_rel_abun=True):
@@ -710,15 +701,15 @@ def contrib_to_legacy(infiles, outfile, use_rel_abun=True):
                              dtype={'function': str, 'sequence': str,
                                     'sample': str, 'taxon': str})
 
-    contrib_df.rename(columns={'sample' : 'Sample',
-                               'function' : 'Gene',
-                               'taxon' : 'OTU',
-                               'genome_function_count' : 'GeneCountPerGenome'},
+    contrib_df.rename(columns={'sample': 'Sample',
+                               'function': 'Gene',
+                               'taxon': 'OTU',
+                               'genome_function_count': 'GeneCountPerGenome'},
                       inplace=True)
 
     if 'norm_taxon_function_contrib' in contrib_df:
-        contrib_df.rename(columns={'norm_taxon_function_contrib' : \
-                                   'ContributionPercentOfSample'},
+        contrib_df.rename(columns={'norm_taxon_function_contrib': \
+                                       'ContributionPercentOfSample'},
                           inplace=True)
 
     abun_col_counter = 0
@@ -740,16 +731,16 @@ def contrib_to_legacy(infiles, outfile, use_rel_abun=True):
                  '\'taxon_rel_function_abun\'.')
     else:
         if use_rel_abun:
-            contrib_df.rename(columns={'taxon_rel_abun' : \
-                                       'OTUAbundanceInSample',
-                                       'taxon_rel_function_abun' : \
-                                       'CountContributedByOTU'}, inplace=True)
+            contrib_df.rename(columns={'taxon_rel_abun': \
+                                           'OTUAbundanceInSample',
+                                       'taxon_rel_function_abun': \
+                                           'CountContributedByOTU'}, inplace=True)
             contrib_df.drop(labels=['taxon_abun', 'taxon_function_abun'],
                             axis=1, inplace=True)
         else:
-            contrib_df.rename(columns={'taxon_abun' : 'OTUAbundanceInSample',
-                                       'taxon_function_abun' : \
-                                       'CountContributedByOTU'}, inplace=True)
+            contrib_df.rename(columns={'taxon_abun': 'OTUAbundanceInSample',
+                                       'taxon_function_abun': \
+                                           'CountContributedByOTU'}, inplace=True)
             contrib_df.drop(labels=['taxon_rel_abun',
                                     'taxon_rel_function_abun'],
                             axis=1, inplace=True)
@@ -760,7 +751,8 @@ def contrib_to_legacy(infiles, outfile, use_rel_abun=True):
                                  'ContributionPercentOfSample']]
 
     contrib_df.to_csv(path_or_buf=outfile, sep="\t", index=False,
-                              compression="gzip")
+                      compression="gzip")
+
 
 def restricted_float(in_arg):
     '''Custom argparse type to force an input float to be between 0 and 1.'''
@@ -768,7 +760,7 @@ def restricted_float(in_arg):
         in_arg = float(in_arg)
     except ValueError:
         raise argparse.ArgumentTypeError(in_arg + " is not a floating-point "
-                                         "literal (i.e. not a proportion)")
+                                                  "literal (i.e. not a proportion)")
 
     if in_arg < 0.0 or in_arg > 1.0:
         raise argparse.ArgumentTypeError(in_arg + "is not in range 0.0 - 1.0")
@@ -809,7 +801,6 @@ class TemporaryDirectory(object):
 
     def cleanup(self):
         if self._finalizer.detach():
-
             # Line added by Gavin Douglas to change permissions to 777 before
             # deleting:
             call(["chmod", "-R", "777", self.name])
@@ -837,32 +828,36 @@ def shuffle_predictions(input, outdir, rep, seed=None):
 
         pred_table_shuffled['sequence'] = np.random.permutation(pred_table['sequence'].values)
 
-        if(input[-7:] == ".tsv.gz"):
+        if (input[-7:] == ".tsv.gz"):
             outfile = join(outdir, input.replace(".tsv.gz", "_shuf" + str(i) + ".tsv.gz"))
-        elif(input[-4:] == ".tsv"):
+        elif (input[-4:] == ".tsv"):
             outfile = join(outdir, input.replace(".tsv", "_shuf" + str(i) + ".tsv"))
         else:
             outfile = join(outdir, input + "_shuf" + str(i) + ".tsv.gz")
-        
+
         pred_table_shuffled.to_csv(path_or_buf=outfile, sep="\t", index=False,
                                    compression="infer")
 
-def sample_bgc_calculate(m1, m2):
-    '''Calculate BGC distribution of samples
-    based on the feature table and bgc table
-    m1 stands for feature table (dtype:DataFrame)
-    m2 stands for bgc table (dtype:DataFrame)
-    '''
-    m2_transposed = m2.T
-    m3_transposed = m2_transposed.dot(m1)
-    m3_df = m3_transposed.T
-    return m3_df
+
+def sample_bgc_calculate(feature_table, psmpa_result):
+    """Calculate BGC distribution of samples based on the feature table and bgc table.
+
+    Args:
+        feature_table: Feature table derived from clustering
+        psmpa_result: BGCs distribution of each sequence obtained by psmpa analysis
+
+    Returns:
+        A dataframe of sample BGCs distribution
+    """
+
+    sample_bgc_distribution = feature_table.T.dot(psmpa_result)
+    return sample_bgc_distribution
 
 
 def get_version(rel_path):
     '''get version information from __init__.py file'''
     with open(rel_path) as fp:
-        cont = fp.read().splitlines()     
+        cont = fp.read().splitlines()
     for line in cont:
         if line.startswith('__version__'):
             delim = '"' if '"' in line else "'"
@@ -872,13 +867,32 @@ def get_version(rel_path):
 
 
 def create_output_folder(output_folder):
-    '''Check whether the folder exists.
-    If not, create a folder.
-    If exists, raise a error.
-    '''
+    """Check whether the folder exists.
+
+    If not, create a folder. If exists, raise a error.
+
+    Args:
+        output_folder: A folder used to store the output.
+
+    Returns:
+        A file successfully created or an error reported.
+    """
 
     try:
         os.makedirs(output_folder)
     except:
-        raise Exception('Stopping since output directory ' + output_folder +
-                        ' already exists.') 
+        raise Exception('Stopping since output directory ' + output_folder + ' already exists.')
+
+
+def get_query_seq_id(query):
+    """Read the input fasta file and get the sequence id.
+    Args:
+        query: Input fasta file.
+    Returns:
+        A series of query sequence ids.
+    """
+    qseqid_ls = []
+    for seq_record in SeqIO.parse(query, "fasta"):
+        qseqid_ls.append(seq_record.id)
+    qseqid = pd.Series(qseqid_ls, name='qseqid')
+    return qseqid
